package demo.composite.mvvm_pattern;

import demo.data.ContributorData;
import demo.data.pojo.Contributor;
import org.zkoss.bind.annotation.Command;
import org.zkoss.bind.annotation.Init;
import org.zkoss.bind.annotation.NotifyChange;

import java.util.ArrayList;
import java.util.List;

public class ContributorVM {

	private List<String> availTitles;
	private List<Contributor> allContributors;

	// Used for binding with selected data,
	private Contributor selectedContributor;
	// A temporary variable which is used for previewing and editing contributor
	private Contributor previewContributor;
	// Default is doing selection
	private boolean addingNewItemMode = false;

	@Init
	public void init() {
		// Preparing data
		availTitles = new ArrayList<String>(new ContributorData().getTitles());
		allContributors = new ArrayList<Contributor>(new ContributorData().getContributors());
		// Select the first item as default
		setSelectedContributor(allContributors.get(0));
		copyToTemp(selectedContributor);
	}

	@Command("select")
	@NotifyChange({ "previewContributor", "addingNewItemMode" })
	public void selectContributor() {
		copyToTemp(selectedContributor);
		setAddingNewItemMode(false);
	}

	@Command("create")
	@NotifyChange({ "previewContributor", "addingNewItemMode" })
	public void createNewContributor() {
		setPreviewContributor(getInitializedContributor());
		setAddingNewItemMode(true);
	}

	@Command("preview")
	@NotifyChange("previewContributor")
	public void previewEditingContributor() {
	}

	@Command("save")
	@NotifyChange({ "contributorList", "addingNewItemMode" })
	public void addNewContributor() {
		getContributorList().add(previewContributor);
		setSelectedContributor(previewContributor);
		setAddingNewItemMode(false);
	}

	@Command("update")
	@NotifyChange({ "selectedContributor", "previewContributor" })
	public void updateSelectedContributor() {
		saveToSelected(previewContributor);
	}

	@Command("delete")
	@NotifyChange({ "selectedContributor", "contributorList", "previewContributor", "addingNewItemMode" })
	public void removeSelectedContributor() {
		getContributorList().remove(selectedContributor);
		setSelectedContributor(null);
		setPreviewContributor(getInitializedContributor());
		setAddingNewItemMode(true);
	}

	/* View Model API */
	public List<String> getContributorTitles() {
		return availTitles;
	}

	public List<Contributor> getContributorList() {
		return allContributors;
	}

	public void setSelectedContributor(Contributor selected) {
		this.selectedContributor = selected;
	}

	public Contributor getSelectedContributor() {
		return selectedContributor;
	}

	public Contributor getPreviewContributor() {
		return previewContributor;
	}

	public void setPreviewContributor(Contributor previewContributor) {
		this.previewContributor = previewContributor;
	}

	public boolean isAddingNewItemMode() {
		return addingNewItemMode;
	}

	public void setAddingNewItemMode(boolean addingNewItemMode) {
		this.addingNewItemMode = addingNewItemMode;
	}

	private Contributor getInitializedContributor() {
		return new Contributor("", "", getContributorTitles().get(0) + "", 0);
	}

	private void saveToSelected(Contributor contributor) {
		selectedContributor.setFirstName(contributor.getFirstName());
		selectedContributor.setLastName(contributor.getLastName());
		selectedContributor.setTitle(contributor.getTitle());
		selectedContributor.setExtension(contributor.getExtension());
	}

	private void copyToTemp(Contributor contributor) {
		previewContributor = new Contributor(contributor.getFirstName(), contributor.getLastName(),
				contributor.getTitle(), contributor.getExtension());
	}
}
